package com.stryker.cmf.accountmgrbean;

import javax.ejb.Local;

/**
 * 
 * @author Anthony Mattas
 *
 */
@Local
public interface AccountMgr {
	/**
	 * Adds a user to the user database, puts them in a default group, and e-mails a randomly generated password
	 * @param username username to add
	 * @param eMail e-mail address to send temporary password to
	 * @throws AccountMagrException 
	 */
	public void addUser(String username, String firstname, String lastname, String title, String phone, String extension, String email) throws AccountMgrException;
	
	/**
	 * Edit the group a user is in.
	 * @param username Username to edit
	 * @param group Group to use
	 * @throws AccountMgrException 
	 */
	public void setGroup(String username, String group) throws AccountMgrException;
	
	/**
	 * Change the users password to a new chosen password.
	 * @param username
	 * @param password
	 * @throws AccountMgrException
	 */
	public void setPassword(String username, String password) throws AccountMgrException;
	
	/**
	 * Changes the user information
	 * @param username
	 * @param firstname
	 * @param lastname
	 * @param title
	 * @param phone
	 * @param extension
	 * @param email
	 */
	public void setInfo(String username, String firstname, String lastname, String title, String phone, String extension, String email) throws AccountMgrException;
	
	/**
	 * Changes the user information
	 * @param user
	 * @throws AccountMgrException
	 */
	public void setInfo(UserInfo user) throws AccountMgrException;
	
	/**
	 * Generates a new password in the case the user has forgotten his.
	 * @param username
	 * @throws AccountMgrException
	 */
	public void newPassword(String username) throws AccountMgrException; 
	
	/**
	 * Returns true or false if a user exists.
	 * @param username
	 * @throws AccountMgrException
	 */
	public Boolean userExists(String username) throws AccountMgrException;
	
	/**
	 * Toggles the disabled flag of an account
	 * @param username
	 * @throws AccountMgrException
	 */
	public void toggleDisabled(String username);
	
}
