package com.stryker.cmf.accountmgrbean;

import java.math.BigInteger;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Random;

import javax.ejb.EJB;
import javax.ejb.Stateless;

import com.stryker.cmf.cimailer.CIMailer;
import com.stryker.cmf.cimailer.CIMailerException;
/**
 * 
 * @author tony.mattas
 *
 */
@Stateless
public class AccountMgrBean implements AccountMgr {
	@EJB UsersFacadeLocal userfacade;
	@EJB CIMailer mailer;
	@EJB UserInfo info;
	
	public void addUser(String username, String firstname, String lastname, String title, String phone, String extension, String email) throws AccountMgrException  {
		String password = randomPassword();
		Users user = userfacade.findById(username);
		if (user != null) { 
			throw new AccountMgrException("The Account " + user.getUsername() + "already exists.");
		}
		if ((lastname == null) || (firstname == null) || (phone == null) || (email == null))
			throw new AccountMgrException("Unable to create user, Missing Critical Data.");
		user = new Users();
		user.setUsername(username);
		user.setGroupname("users");
		user.setFirstname(firstname);
		user.setLastname(lastname);
		user.setTitle(title);
		user.setPhone(phone);
		user.setExtension(extension);
		user.setEmail(email);
		user.setDisabled(0);
		user.setNumfails(0);
		try {
			user.setPassword(hash(password));
			userfacade.save(user);
			mailer.addRecipient(email);
			mailer.setSubject("Stryker CranioMaxilloFacial Login Information");
			mailer.setSender("CIAdministrators@stryker.com", "Stryker CranioMaxilloFacial");
			mailer.setMessage("Dear User,\n\n" +
					"Welcome to the Stryker CranioMaxilloFacial custom implant site. This e-mail contains your registration information " +
					"along with a temporary username and password. \n\n" +
					"Username: " + username + "\n" +
					"Password: " + password + "\n\n" +
					"You will be asked to change this password on first login, however please note that you will not be allowed " +
					"to log in until your registration request is processed by a Stryker CranioMaxilloFacial representative. " +
					"You will receive a second e-mail when this process is completed.\n\n" +
					"The above username and password is for the following person:\n\n" +
					user.getFirstname() + " " + user.getLastname() + "\n" +
					"(" + user.getPhone().substring(0, 3) + ") " + user.getPhone().substring(3, 6) + "-" + 
					user.getPhone().substring(6, 10) + " " + user.getExtension() + "\n" +
					user.getEmail() + "\n\n" + 
					" If this is not you please delete this e-mail and immediately contact 1+ (269) 324-5346.");

			mailer.sendMessage();
		} catch (NoSuchAlgorithmException e1) {
			throw new AccountMgrException("MD5 Algorithm Does Not Exist.", e1);
		} catch (RuntimeException e) {
			throw new AccountMgrException("Unable to Commit Data.", e);
		} catch (CIMailerException e) {
			throw new AccountMgrException("Unable to mail password.", e); 
		}
	}
	
	/**
	 * Generates a MD5 Hash
	 * @param password Plain text password
	 * @return Hashed Password
	 * @throws NoSuchAlgorithmException 
	 * @throws NoSuchAlgorithmException
	 */
	private String hash(String password) throws NoSuchAlgorithmException {
		/* String to return */
		String hashed = null;
		BigInteger hash;
		try {
			/* Hashing algorithm */
			MessageDigest md5 = MessageDigest.getInstance("MD5");
			md5.update(password.getBytes());
			hash = new BigInteger(1, md5.digest());
			hashed = hash.toString(16);
			/* Acount for varying string lengths */
			if (hashed.length() < 32) {
				hashed = 0 + hashed;
			}
		} catch (NoSuchAlgorithmException e) {
			/* Throw up to parent */
			throw e;
		}
		return hashed;
	}

	/**
	 * Generates a random password
	 * @return The new Random password
	 */
	private String randomPassword() {
		char[] allowedChars = "abcdefghijklmnopqrstuvqxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890".toCharArray();
		Random randNumberGen = new Random();
		StringBuffer password = new StringBuffer();
		int randNumber;
		for (int i=0; i<10; i++) {
			randNumber = (randNumberGen.nextInt(61));
			password.append(allowedChars[randNumber]);
		}
		return password.toString();
	}
	
	public void setGroup(String user, String group) throws AccountMgrException {
		Users username = new Users();
		username = userfacade.findById(user);
		// Check to see if the user exists.
		if (username == null) {
			throw new AccountMgrException("User " + user + " Does Not Exist.");
		}
		if (group == null) {
			throw new AccountMgrException("Can't set null group.");
		}
		else {
			//Edit group
			username.setGroupname(group);
			try {
				userfacade.update(username);
			} catch (RuntimeException e) {
				throw new AccountMgrException("Unable to Commit Data.", e);
			}	
		}
	}
	
	public void setPassword(String username, String password) throws AccountMgrException {
		Users user = new Users();
		user = userfacade.findById(username);
		// Check to see if the user exists.
		if (user == null) {
			throw new AccountMgrException("User " + username + " Does Not Exist.");
		}
		if (password == null) {
			throw new AccountMgrException("Can't set null password");
		}
		else {
			//Edit group
			try {
				user.setPassword(hash(password));
				userfacade.update(user);
			} catch (RuntimeException e) {
				throw new AccountMgrException("Unable to Commit Data.", e);
			} catch (NoSuchAlgorithmException e) {
				throw new AccountMgrException("MD5 Algorithm Does Not Exist.", e);
			}
		}
	}
	
	public void newPassword(String username) throws AccountMgrException {
		setPassword(username, randomPassword());
	}
	
	public void setInfo(String username, String firstname, String lastname, String title, String phone, String extension, String email) throws AccountMgrException {
		Users user = userfacade.findById(username);
		if (user == null) { 
			throw new AccountMgrException("The Account " + user.getUsername() + "does not exist.");
		}
		if ((lastname == null) || (firstname == null) || (phone == null) || (email == null))
			throw new AccountMgrException("Unable to Commit Data, Missing Critical Data.");
		user.setUsername(username);
		if (user.getGroupname() == null) user.setGroupname("users");
		user.setFirstname(firstname);
		user.setLastname(lastname);
		user.setTitle(title);
		user.setPhone(phone);
		user.setExtension(extension);
		user.setEmail(email);
		userfacade.save(user);
	}
	
	public void setInfo(UserInfo user) throws AccountMgrException {
		setInfo(user.getUsername(), user.getFirstname(), user.getLastname(), user.getTitle(), user.getPhone(), user.getExtension(), user.getEmail());
	}
	
	public Boolean userExists(String username) throws AccountMgrException {
		if (userfacade.findById(username) == null) return false;
		else return true;
	}
	
	public void toggleDisabled(String username) {
		System.out.println("***1***");
		Users user = userfacade.findById(username);
		System.out.println("***2***");
		if (user.getDisabled() == 0) {user.setDisabled(1);
		System.out.println("***3***");}
		else user.setDisabled(0);
		System.out.println("***4***");
		userfacade.save(user);
	}
}
	

