package com.stryker.cmf.accountmgrbean;

import java.util.ArrayList;
import java.util.Date;

import javax.ejb.Local;

@Local
/**
 * Interface for user info, used for passing information between 
 * entity and session beans, granted I could have directly used the session bean
 * using this local interface gives me an extra layer of control
 */
public interface UserInfo {

	/**
	 * Getter for username
	 * @return
	 */
	public String getUsername();
	
	/**
	 * Setter for Username
	 * @param username 
	 */
	public void setUsername(String username);
	
	/**
	 * Getter for groupname
	 * @return
	 */
	public String getGroupname();
	
	/**
	 * Setter for Groupname
	 * @param groupname
	 */
	public void setGroupname(String groupname);

	/**
	 * Getter for Firstname
	 * @return
	 */
	public String getFirstname();

	/**
	 * Setter for Firstname
	 * @param firstname
	 */
	public void setFirstname(String firstname);
	
	/**
	 * Getter for lastname
	 * @return
	 */
	public String getLastname();
	
	/**
	 * Setter for lastname
	 * @param lastname
	 */
	public void setLastname(String lastname);
	
	/**
	 * Getter for title
	 * @return
	 */
	public String getTitle();
	
	/**
	 * Setter for title
	 * @param title
	 */
	public void setTitle(String title);
	
	/**
	 * Getter for phone
	 * @return
	 */
	public String getPhone();
	
	/**
	 * Setter for phone
	 * @param phone
	 */
	public void setPhone(String phone);
	
	/**
	 * Getter for extension
	 * @return
	 */
	public String getExtension();
	
	/**
	 * Setter for extension
	 * @param extension
	 */
	public void setExtension(String extension);
	
	/**
	 * Getter for email
	 * @return
	 */
	public String getEmail();
	
	/**
	 * Setter for email
	 * @param email
	 */
	public void setEmail(String email);
	
	/**
	 * Returns a list of all users
	 * @return ArrayList of Users
	 * @throws AccountMgrException
	 */
	public ArrayList<UserInfo> allUsers() throws AccountMgrException;

	/**
	 * Returns a user object
	 * @param username The username to retrieve
	 * @return The user object returned
	 * @throws AccountMgrException
	 */
	public UserInfo getInfo(String username) throws AccountMgrException;
	
	/**
	 * Returns last successful login attempt
	 * @return Date formatted last success
	 */
	public Date getLastsuccess();

	/**
	 * Setter for last success, does not update database if object saved
	 * @param lastsuccess
	 */
	public void setLastsuccess(Date lastsuccess);

	/**
	 * Get number of failures
	 * @return Integer representing number of failures
	 */
	public Integer getNumfails();

	/**
	 * Setter for number of failures, does not update database if object saved
	 * @param numfails Integer for number of failures
	 */
	public void setNumfails(Integer numfails);

	/**
	 * Get disabled status of account
	 * @return Integer for disabled 
	 */
	public Integer getDisabled();

	/**
	 * Setter for disabled flag, does not update database if object saved
	 * @param disabled
	 */
	public void setDisabled(Integer disabled);
	
	/**
	 * Call's the AccountMgr toggleDisable function. This has to be here so it can be
	 * called as a table object.
	 * @return The string refresh which will be handled by faces.
	 */
	public String toggleDisabled();

	
}
