package com.stryker.cmf.cimailer;

import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.Date;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import javax.ejb.EJB;
import javax.ejb.Stateless;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Multipart;
import javax.mail.Transport;
import javax.mail.internet.AddressException;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeMultipart;

import com.stryker.cmf.civalidators.CIValidator;

/**
 * 
 * @author Anthony G. Mattas
 *
 */
@Stateless
public class CIMailerBean implements CIMailer {

	@Resource(name="comp/env/mail/CIMailer") private javax.mail.Session session;
	@EJB CIValidator validator;
	
	private String subject;
	private StringBuffer message;
	/* Hash maps for e-mail address in the format Address, Name */
	private ArrayList<String> recipient;
	private ArrayList<String> cc;
	private ArrayList<String> bcc;
	private String senderaddress;
	private String sendername;
	
	/***
	 *
	 */
	@SuppressWarnings("unused")
	@PostConstruct
	private void init() {
		this.message = new StringBuffer();
		this.recipient = new ArrayList<String>();
		this.cc = new ArrayList<String>();
		this.bcc = new ArrayList<String>();
	}
		
	/**
	 * Add a recipient to the message
	 * @param address
	 * @return Number of Recipients in class
	 * @throws CIMailerException
	 */
	public int addRecipient(String address) throws CIMailerException {
		if (validator.email(address) == false) 
			throw new CIMailerException("Invalid E-mail Address Format");
		recipient.add(address);
		return recipient.size();
	}
	
	/**
	 * Add a recipient to the message
	 * @param address
	 * @return Number of Recipients in class
	 * @throws CIMailerException
	 */
	public int addCC(String address) throws CIMailerException {
		if (validator.email(address) == false) 
			throw new CIMailerException("Invalid E-mail Address Format");
		cc.add(address);
		return recipient.size();
	}
	
	/**
	 * Add a recipient to the message
	 * @param address
	 * @return Number of Recipients in class
	 * @throws CIMailerException
	 */
	public int addBCC(String address) throws CIMailerException {
		if (validator.email(address) == false) 
			throw new CIMailerException("Invalid E-mail Address Format");
		bcc.add(address);
		return recipient.size();
	}

	/**
	  * Sets the Subject
	  * @param subject
	  * @throws CIMailerException
	  */
	 public void setSubject(String subject) throws CIMailerException {
		 if (subject == null) throw new CIMailerException("Subject is Null");
		 this.subject = subject;
	 }
	 
	/**
	 * Sets the sender address
	 * @param senderaddress
	 * @param sendername
	 * @throws CIMailerException
	 */
	 public void setSender(String senderaddress, String sendername) throws CIMailerException {
			if (validator.email(senderaddress) == false) 
				throw new CIMailerException("Invalid E-mail Address Format");
			else {
				this.sendername = sendername;
				this.senderaddress = senderaddress;
			}
	 }
	 
	 /**
	  * Sets the message to be send
	  * @param message
	  * @return Message length
	  */
	 public int setMessage(String message) {
		 this.message = new StringBuffer(message);
		 return this.message.length();
	 }
	 
	 /**
	  * Appends the message being sent
	  * @param message
	  * @return Message length
	  */
	 public int appendMessage(String message) {
		 this.message.append(message);
		 return this.message.length();
	 }
	 
	 /** 
	  * Sends a message
	  * @throws CIMailerException 
	  */
	 public void sendMessage() throws CIMailerException {
		 try {
			 Message msg = new MimeMessage(session);
			 if ((this.sendername != null) && (this.senderaddress != null))
				 msg.setFrom(new InternetAddress(this.senderaddress, this.sendername));
			 if (this.subject != null)
				 msg.setSubject(this.subject);
			 else 
				 throw new CIMailerException("Subject is Null");
			 // Not allowing date spoofing
			 msg.setSentDate(new Date());
			 msg.setFrom();
			 if ((this.recipient.size() < 1) && (this.cc.size() < 1) && (this.bcc.size() < 1)) 
				 throw new CIMailerException("No Recipients");
			 for (String address: this.recipient) {
				 msg.addRecipients(Message.RecipientType.TO, InternetAddress.parse(address, false));			 
			 }
			 for (String address: this.cc) {
				 msg.addRecipients(Message.RecipientType.CC, InternetAddress.parse(address, false));			 
			 }
			 for (String address: this.bcc) {
				 msg.addRecipients(Message.RecipientType.BCC, InternetAddress.parse(address, false));			 
			 }
			 MimeBodyPart mbp = new MimeBodyPart();
			 if (this.message.toString() != null)
				 mbp.setText(this.message.toString());
			 Multipart mp = new MimeMultipart();
			 mp.addBodyPart(mbp);
			 msg.setContent(mp);
			 Transport.send(msg);
		} catch (AddressException e) {
			throw new CIMailerException("Internal Address Exception", e);
		} catch (MessagingException e) {
			throw new CIMailerException("Internal Messaging Exception", e);
		} catch (UnsupportedEncodingException e) {
			throw new CIMailerException("Unsupported encoding type", e);
		}
	}
}

