package com.stryker.cmf.helpers;

import javax.ejb.EJB;
import javax.faces.component.UIInput;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.servlet.http.HttpSession;

import com.stryker.cmf.accountmgrbean.AccountMgr;
import com.stryker.cmf.accountmgrbean.AccountMgrException;
import com.stryker.cmf.accountmgrbean.UserInfo;


/**
 * Backing bean that provides authenticated user tools to faces pages
 * @author tony.mattas
 *
 */
public class AuthTools extends UIInput {
	@EJB AccountMgr userMgr;
	@EJB UserInfo user;

	/**
	 * Is a user logged in
	 * @return true/false
	 */
	public Boolean getLoggedIn() {
		ExternalContext externalContext = FacesContext.getCurrentInstance().getExternalContext();
		return ((externalContext.getUserPrincipal() != null) ? true : false);
	}
	
	/**
	 * Is a user a doctor
	 * @return true/false
	 */
	public Boolean getIsDoctor() {
		ExternalContext externalContext = FacesContext.getCurrentInstance().getExternalContext();
		return (externalContext.isUserInRole("doctor") ? true : false);
	}
	
	/**
	 * Is a user an admin
	 * @return true/false
	 */
	public Boolean getIsAdmin() {
		ExternalContext externalContext = FacesContext.getCurrentInstance().getExternalContext();
		return (externalContext.isUserInRole("admin") ? true : false);
	}

	/**
	 * Is a user a regular user
	 * @return true/false
	 */
	public Boolean getIsUser() {
		ExternalContext externalContext = FacesContext.getCurrentInstance().getExternalContext();
		return (externalContext.isUserInRole("users") ? true : false);
	}
	
	/**
	 * Is a user an account manager
	 * @return
	 */
	public Boolean getIsAcctMgr() {
		ExternalContext externalContext = FacesContext.getCurrentInstance().getExternalContext();
		return (externalContext.isUserInRole("acctmgr") ? true : false);
	}
	
	/**
	 * Is the user in the sales force
	 * @return true/false
	 */
	public Boolean getIsSales() {
		ExternalContext externalContext = FacesContext.getCurrentInstance().getExternalContext();
		return (externalContext.isUserInRole("sales") ? true : false);
	}
	
	/**
	 * Is the user in house
	 * @return true/false
	 */
	public Boolean getIsInHouse() {
		ExternalContext externalContext = FacesContext.getCurrentInstance().getExternalContext();
		return (externalContext.isUserInRole("inhouse") ? true : false);
	}
	
	/**
	 * What is the users firstname
	 * @return
	 */
	public String getFirstName() {
		String username;
		ExternalContext externalContext = FacesContext.getCurrentInstance().getExternalContext();
		username = externalContext.getUserPrincipal().getName();
		if (username != null) {
			try {
				return user.getInfo(username).getFirstname();
			} catch (AccountMgrException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		return "Unknown";
	}

	/**
	 * What is the users last name
	 * @return
	 */
	public String getLastName() {
		String username;
		ExternalContext externalContext = FacesContext.getCurrentInstance().getExternalContext();
		username = externalContext.getUserPrincipal().getName();
		if (username != null) {
			try {
				return user.getInfo(username).getLastname();
			} catch (AccountMgrException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		return "User";
	}
	
	public String logOut() {
		HttpSession session = (HttpSession)FacesContext.getCurrentInstance().getExternalContext().getSession(false);
		session.invalidate();
		return "logout";
	}
}
